package dev.fyloz.colorrecipesexplorer.service.users

import dev.fyloz.colorrecipesexplorer.config.security.defaultGroupCookieName
import dev.fyloz.colorrecipesexplorer.model.account.*
import dev.fyloz.colorrecipesexplorer.repository.GroupRepository
import dev.fyloz.colorrecipesexplorer.service.AbstractExternalNamedModelService
import dev.fyloz.colorrecipesexplorer.service.ExternalNamedModelService
import org.springframework.context.annotation.Profile
import org.springframework.stereotype.Service
import org.springframework.web.util.WebUtils
import javax.servlet.http.HttpServletRequest
import javax.servlet.http.HttpServletResponse
import javax.transaction.Transactional

const val defaultGroupCookieMaxAge = 10 * 365 * 24 * 60 * 60 // 10 ans

interface GroupService :
    ExternalNamedModelService<Group, GroupSaveDto, GroupUpdateDto, GroupOutputDto, GroupRepository> {
    /** Gets all the users of the group with the given [id]. */
    fun getUsersForGroup(id: Long): Collection<User>

    /** Gets the default group from a cookie in the given HTTP [request]. */
    fun getRequestDefaultGroup(request: HttpServletRequest): Group

    /** Sets the default group cookie for the given HTTP [response]. */
    fun setResponseDefaultGroup(groupId: Long, response: HttpServletResponse)
}

@Service
@Profile("!emergency")
class GroupServiceImpl(
    private val userService: UserService,
    groupRepository: GroupRepository
) : AbstractExternalNamedModelService<Group, GroupSaveDto, GroupUpdateDto, GroupOutputDto, GroupRepository>(
    groupRepository
),
    GroupService {
    override fun idNotFoundException(id: Long) = groupIdNotFoundException(id)
    override fun idAlreadyExistsException(id: Long) = groupIdAlreadyExistsException(id)
    override fun nameNotFoundException(name: String) = groupNameNotFoundException(name)
    override fun nameAlreadyExistsException(name: String) = groupNameAlreadyExistsException(name)

    override fun Group.toOutput() = GroupOutputDto(
        this.id!!,
        this.name,
        this.permissions,
        this.flatPermissions
    )

    override fun existsByName(name: String): Boolean = repository.existsByName(name)
    override fun getUsersForGroup(id: Long): Collection<User> =
        userService.getByGroup(getById(id))

    @Transactional
    override fun save(entity: Group): Group {
        return super<AbstractExternalNamedModelService>.save(entity).apply {
            userService.saveDefaultGroupUser(this)
        }
    }

    override fun update(entity: GroupUpdateDto): Group {
        val persistedGroup by lazy { getById(entity.id) }
        return update(with(entity) {
            Group(
                entity.id,
                if (name.isNotBlank()) entity.name else persistedGroup.name,
                if (permissions.isNotEmpty()) entity.permissions else persistedGroup.permissions
            )
        })
    }

    @Transactional
    override fun delete(entity: Group) {
        userService.delete(userService.getDefaultGroupUser(entity))
        super.delete(entity)
    }

    override fun getRequestDefaultGroup(request: HttpServletRequest): Group {
        val defaultGroupCookie = WebUtils.getCookie(request, defaultGroupCookieName)
            ?: throw NoDefaultGroupException()
        val defaultGroupUser = userService.getById(
            defaultGroupCookie.value.toLong(),
            ignoreDefaultGroupUsers = false,
            ignoreSystemUsers = true
        )
        return defaultGroupUser.group!!
    }

    override fun setResponseDefaultGroup(groupId: Long, response: HttpServletResponse) {
        val group = getById(groupId)
        val defaultGroupUser = userService.getDefaultGroupUser(group)
        response.addHeader(
            "Set-Cookie",
            "$defaultGroupCookieName=${defaultGroupUser.id}; Max-Age=$defaultGroupCookieMaxAge; Path=/api; HttpOnly; Secure; SameSite=strict"
        )
    }
}
