package dev.fyloz.colorrecipesexplorer.service.files

import io.mockk.clearAllMocks
import io.mockk.every
import io.mockk.mockk
import io.mockk.spyk
import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.assertThrows
import org.springframework.core.io.Resource
import org.springframework.core.io.ResourceLoader
import kotlin.test.assertEquals
import kotlin.test.assertFalse
import kotlin.test.assertTrue

class ResourceFileServiceTest {
    private val resourceLoader = mockk<ResourceLoader>()

    private val service = spyk(ResourceFileService(resourceLoader))

    @AfterEach
    fun afterEach() {
        clearAllMocks()
    }

    private fun existsTest(shouldExists: Boolean, test: (String) -> Unit) {
        val path = "unit_test_resource"
        with(service) {
            every { path.fullPath() } returns mockk {
                every { resource } returns mockk {
                    every { exists() } returns shouldExists
                }
            }

            test(path)
        }
    }

    @Test
    fun `exists() returns true when a resource exists at the given path`() {
        existsTest(true) { path ->
            val found = service.exists(path)

            assertTrue { found }
        }
    }

    @Test
    fun `exists() returns false when no resource exists at the given path`() {
        existsTest(false) { path ->
            val found = service.exists(path)

            assertFalse { found }
        }
    }

    private fun readTest(shouldExists: Boolean, test: (Resource, String) -> Unit) {
        val mockResource = mockk<Resource> {
            every { exists() } returns shouldExists
        }
        val path = "unit_test_path"
        with(service) {
            every { path.fullPath() } returns mockk {
                every { resource } returns mockResource
            }

            test(mockResource, path)
        }
    }

    @Test
    fun `read() returns the resource at the given path`() {
        readTest(true) { resource, path ->
            val found = service.read(path)

            assertEquals(resource, found)
        }
    }

    @Test
    fun `read() throws FileNotFoundException when no resource exists at the given path`() {
        readTest(false) { _, path ->
            assertThrows<FileNotFoundException> {
                service.read(path)
            }
        }
    }

    @Test
    fun `fullPath() returns the given path in the classpath`() {
        val path = "unit_test_path"
        val expectedPath = "classpath:$path"

        with(service) {
            val found = path.fullPath()

            assertEquals(expectedPath, found.path)
        }
    }

    @Test
    fun `resource returns a resource for the given path`() {
        val filePath = FilePath("classpath:unit_test_path")
        val resource = mockk<Resource>()

        every { resourceLoader.getResource(filePath.path) } returns resource

        with(service) {
            val found = filePath.resource

            assertEquals(resource, found)
        }
    }
}
